****** Make Friends with SAS, Day 2: Working with Variables;
****** Created by Lesa Hoffman;

*******************************************************************************
*******						MAKING OUTPUT LOOK NICE				     	*******
******************************************************************************;

* Setting global system options for formatting output:
	center/nocenter, date/nodate, number/nonumber for output pages,
	linesize = number of lines across (64-256 or MAX), 
	pagesize = #lines per page (15-32767 or MAX),
	formdlim = ' ' eliminates page breaks in output, 
			 = '' to put them back
			 = '-' puts a line between outputs  (can use with any character) 
    Initial TITLE shuts off annoying SAS default headers;
OPTIONS nocenter nodate nonumber linesize=120 pagesize=MAX formdlim='-';
TITLE;

* Titles stay in effect until next one or until just TITLE. 
  You can use single or double quotes, but double are more versatile;
TITLE1 "This is my first title";
TITLE2 "This is my second title";	* Can have up to 9 titles;
TITLE1; TITLE2;		* Previous titles are now shut off;


*** DEFINE NAME OF DATAFILE ONCE;

* We can use a macro variable to name our dataset --
  then, if you change your data set, you only need to change it ONCE;
%LET datafile=work.yourfilename;

* Then, you can use the macro variable name instead throughout;
* To refer to a macro variable, start with an "&" and end with a period;
DATA &datafile.; SET &datafile.; 
	* Transformations below happen here...;
RUN;

* Macro variables can be put in ANYTHING: titles, labels, variable names, etc;


*******************************************************************************
*******				NUMERIC VARIABLE TRANSFORMATIONS					*******
******************************************************************************;

****************** Critical Point ******************************;
* Transformations must happen within a DATA step: use DATA + SET;
* If you forget the SET, you will create an empty file!;
****************************************************************;

* These transformations can be used if all values necessary are in the same row
  If summarizing across rows, you'll need PROC MEANS BY ID instead (see day 1);

* IF-THEN allows for conditional processing
  Make sure to put " " around string variable characters
  Use parentheses to make sure expression is read as intended
  "IF" gets paired with other relationsal operators such as:
	"equal to"					=	EQ
 	"not equal to"				~=	NE	
	"less than"					<	LT
	"less than or equal to"		<= 	LE
	"greater than"				>	GT
	"greater than or equal to"	>=	GE

	"and"						&	AND
	"or"						|	OR
	"not"						~	NOT	;

* There is no RECODE in SAS - you have to use IF-THEN,ELSE yourself
  You can compute new variables, but you don't need the word COMPUTE;

* IF-THEN,ELSE-THEN works for single transformations
* Need IF-THEN-DO-END,ELSE-DO-END for multiple transformations;

* Transformations must happen within a DATA step: use DATA + SET
  If you forget the SET, you will create an empty file!;
 DATA &datafile.; SET &datafile.;

* There are different ways to refer to a series of variables
  Dont forget the "OF" or it will try to subtract them;
	meanvar = MEAN(OF var1, var2, var3, var4);	* long way;
	meanvar = MEAN(OF var1-var4);				* contiguous names;
	meanvar = MEAN(OF var1--var4);				* contiguous positions;
	LABEL meanvar= "meanvar: Mean of Var1-Var4";

* Note this is NOT equivalent to the code above
  When using operators, if ANY are missing, the meavar will be missing;
	meanvar = (var1 + var2 + var3 + var4) / 4;
	LABEL meanvar= "meanvar: Mean of Var1-Var4 (only if all 4)";

* NMISS function returns number MISSING rsponses for numeric variables 
  N function returns number of VALID responses for numeric variables;
	nmiss10  = NMISS(OF var1--var10);
	nvalid10 = N(OF var1--var10);
	LABEL nmiss10= "nmiss10: Number Missing for Var1-10"
		  nval10=  "nval10: Number Answered for Var1-10";

* There is no direct way to specify minimum number to be computed;
* NMISS function can count number of missing values and IF to select;
	IF NMISS(OF var1-var4) LE 1 THEN DO; 
		meanvar = MEAN(OF var1-var4); maxvar = MAX(OF var1-var4); END;
	ELSE DO; 
		meanvar = .; maxvar = .; END;
* Otherwise, N function can count number of valid values and IF to select;
	IF N(OF var1-var4) GE 3 THEN DO; 
		meanvar = MEAN(OF var1-var4); maxvar = MAX(OF var1-var4); END;
	ELSE DO; 
		meanvar = .; maxvar = .; END;
	LABEL meanvar= "meanvar: Mean of Var1-Var4 (if at least 3)"
		  maxvar=  "maxvar: Max of Var1-Var4 (if at least 3)";

* Your turn;



 * Example of centering a continuous predictor at 30;
	c_pred = pred - 30;
	LABEL c_pred= "c_pred: Predictor Centered at 30 (0=30)";


* Your turn;



* Hundreds of other possible functions;
* Functions can include variables or numeric constants as needed;
* Here are some more types of transformation;
	newvar = oldvar**2;   			* squaring old variable to make new;
	logvar = LOG(var);  			* taking natural log;
	log10var = LOG10(var);  		* taking log-base 10;
	maxvar = MAX(var);				* maximum value;
	minvar = MIN(var);				* minimum value;
	sumvar = SUM(OF var1, var2);	* sum of old variables;
	stdvar = STD(OF var1, var2);    * standard deviation of variables;

* Example of reverse coding -- specify missing values first;
* To do this for more than one variable at a time, use an ARRAY instead;
	     IF (var1=.) THEN DO; rvar1=.; END;
	ELSE IF (var1=5) THEN DO; rvar1=1; END;
	ELSE IF (var1=4) THEN DO; rvar1=2; END;
	ELSE IF (var1=3) THEN DO; rvar1=3; END;
	ELSE IF (var1=2) THEN DO; rvar1=4; END;
	ELSE IF (var1=1) THEN DO; rvar1=5; END;
	ELSE                  DO; rvar1=.; END;
	LABEL rvar1= "rvar1: Var1 Reverse-Coded";

* Example of creating categories from a continuous variable;
* missing or <1 = missing, 1 to 1.5 = 1, 1.51 to highest = 2;
* The "." = system missing, 999 for unanticipated value;
* Specify missing values first, because in SAS . is < 0;
	     IF var1 = .     					THEN DO; catvar1 = .; END;
	ELSE IF var1 LE 0.99  				  	THEN DO; catvar1 = .; END;
	ELSE IF var1 GT 0.99 AND var1 LE 1.50 	THEN DO; catvar1 = 1; END;
	ELSE IF var1 GT 1.50 				 	THEN DO; catvar1 = 2; END;
	ELSE DO; 									     catvar1 = 999; END;
	LABEL catvar1= "catvar1: Var1 1=1.0-1.5, 2=1.50+, 999=broken";

* Your turn;



******* Other useful functions;

* Saving row number as permanent variable using _N_ keyword;
	IDvar = _N_;
	LABEL IDvar "IDvar: New ID variable";

* The IN function searches a variable for a series of possible values;
* This gives the variable 'pick2468' a value of 1 if var1 = 2 or 4 or 6 or 8;
	IF var1 IN(2,4,6,8,.) THEN pick2468=1; ELSE pick2468=0;
	LABEL pick2468= "pick2468: Var1=2,4,6,or8 (0=no, 1=yes)";

* The ROUND function can be used to round continuous variables;
	roundyr = ROUND(year,10);
	LABEL roundyr= "roundyr: Year Rounded to Nearest Decade";

* Don't forget the "run" that closes the data step;
RUN;


* You can add value labels for new variables in a PROC FORMAT step;
* Here, we create a new format to go with the "catvar1" variable;
PROC FORMAT LIBRARY=work;
	VALUE Fcatvar1 1="1-1.5 of var1" 2=">1.5 of var1";
RUN;
* A FORMAT statement to use this new format would look like this:
	FORMAT catvar1 Fcatvar.;



*******************************************************************************
*******			     WAYS OF DESCRIBING/SUMMARIZING DATA			    *******
******************************************************************************;

* Can use WHERE with any of these to select certain cases only;

* To print sample descriptive statistics for continuous variables;
TITLE1 "Descriptive Statistics for My Continuous Variables";
PROC MEANS N MEAN STDDEV MIN MAX SKEW KURTOSIS DATA=work.datafilename;
	VAR var1 var2 var4-var10;	* Variables to use;
	WHERE group=1;				* Select only these cases, omit for all cases;
RUN; TITLE1;

* To print sample descriptive statistics for continuous variables by group;
TITLE1 "Descriptive Statistics for My Continuous Variables";
PROC MEANS N MEAN STDDEV MIN MAX SKEW KURTOSIS DATA=work.datafilename;
	CLASS group;				* Will print per level of CLASS variables;
	VAR var1 var2 var4-var10;   * Variables to use;
RUN; TITLE1;

* To print frequency tables, where TABLE = variables to use;
TITLE1 "Descriptive Statistics for My Grouping Variables";
PROC FREQ DATA=work.datafile;				
	TABLE catvar1 catvar2 catvar1*catvar2;  * Use of "*" gives crosstab;
	FORMAT catvar1 Fcatvar1.;	* Catvar1 will use format "Fcatvar1" defined above;
RUN; TITLE1;

* PROC UNIVARIATE does all of this and a lot more;
TITLE1 "Descriptive Statistics and Plots";
PROC UNIVARIATE DATA=work.datafile FREQ;	* FREQ = print frequencies;
	CLASS group;				* Will print per level of CLASS variables;
	VAR var1; 	* Variables to provide summary info for;
	* Distribution plots, with estimated shapes requested after;
	  HISTOGRAM var1 / NORMAL LOGNORMAL; 		 	* Overlay these curves and run tests;
	  	INSET NORMAL LOGNORMAL / FORMAT=6.2 POS=NE;	* Put summary stats in a box in northeast corner of plot;
	  QQPLOT var1 / NORMAL(MU=est SIGMA=est); * Quantile-Quantile Plot assuming normal;
	  	INSET NORMAL / FORMAT=6.2;		* Put summary stats in a box in the plot (default is northwest);
	  PROBPLOT var1 / NORMAL(MU=est SIGMA=est); * Probability Plot assuming normal;
	  	INSET NORMAL / FORMAT=6.2;		* Put summary stats in a box in the plot;
	  QQPLOT var1 / LOGNORMAL(THRESHOLD=est SCALE=est SHAPE=est); * Quantile-Quantile Plot assuming lognormal;
		INSET  LOGNORMAL / FORMAT=6.2;	* Put summary stats in a box in the plot;
	  PROBPLOT var1 / LOGNORMAL(THRESHOLD=est SCALE=est SHAPE=est);  * Probability Plot assuming lognormal;
	    INSET  LOGNORMAL / FORMAT=6.2;	* Put summary stats in a box in the plot;
RUN; QUIT; * Need both RUN and QUIT; TITLE1;

* Your turn;





* You will notice that there is no easy way to clear the output window between runs;
* Here is how to do so in code: after whatever you want printed, type this;
* The CLOSE closes the directory, and the second ODS HTML opens it for next time;
* On Day 4 we will see how to direct the output to different places;
ODS HTML CLOSE; ODS HTML;	


*******************************************************************************
******* 				PLOTTING USING PROC GPLOT				    	*******
******************************************************************************;

******************************************************************************
Specifying system options for graphics below using GOPTIONS:
FOR THE GRAPH:
   	RESET=ALL: 			Reset all existing system options for graphics
   	BORDER/NOBORDER: 	Border/No border around graph 
	FTEXT:				Text font in graph
	FTITLE:				Title font
	VSIZE:				LENGTH of total area in inches taken up by the graph 
	HSIZE:				WIDTH of total area in inches taken up by the graph 
 	INTERPOL=JOIN:		Joins points together from same person 
*******************************************************************************;

* Setting generic plotting options;
GOPTIONS RESET=ALL NOBORDER FTEXT="Times New Roman" FTITLE="Times New Roman";
* Example simple scatterplot;
TITLE1 JUSTIFY=CENTER HEIGHT=12pt "This is My Scatterplot";
PROC GPLOT DATA=&datafile.; 
	PLOT varY1*varX;  * Plot Y by X;
	PLOT varY2*varX;  * Can have multiple plots;
RUN; QUIT; TITLE1;
	

* Setting additional system options for graphics below;
GOPTIONS VSIZE=5.5in HSIZE=6.0in INTERPOL=JOIN;

* Generating spaghetti plots of change over time for INDIVIDUALS;
TITLE1 JUSTIFY=CENTER HEIGHT=12pt "This is My Spaghetti Plot"; 
PROC GPLOT DATA=&datafile.; 
* Settings for Y-axis (differ by DV);
 	AXIS1 LENGTH=4.5in LABEL=(ANGLE=90 HEIGHT=12pt "Outcome 1") ORDER=(0 TO 10 BY 1);
	AXIS2 LENGTH=4.5in LABEL=(ANGLE=90 HEIGHT=12pt "Outcome 2") ORDER=(0 TO 100 BY 10);
* Settings for X-axis;
 	AXIS3 LENGTH=4.5in LABEL=(HEIGHT=12pt "My X-Axis Title") ORDER=(1 TO 6 BY 1);
 * Plot Y*X=separate person, start axes at 0 unless otherwise specified;
 	PLOT varY1*varX=ID / VZERO HZERO NOLEGEND VAXIS=AXIS1 HAXIS=AXIS3;  
	PLOT varY2*varX=ID / VZERO HZERO NOLEGEND VAXIS=AXIS2 HAXIS=AXIS3; 
RUN; QUIT; TITLE1;


* Example graph to show lines for different groups;
TITLE1 JUSTIFY=CENTER HEIGHT=1.5 "This is the Title of My Graph";
PROC GPLOT DATA=work.datafilename;
	 * Settings for Y-axis: text height, text orientation, label, size of axis, scale of axis;
	 	AXIS1 LENGTH=4.5in LABEL=(HEIGHT=12pt ANGLE=90 "My Y-Axis") ORDER=(0 TO 10 BY 2);
	 * Settings for X-axis text height, label, size of axis, scale of axis;
	 	AXIS2 LENGTH=4.5in LABEL=(HEIGHT=12pt "My X-Axis") ORDER=(0 TO 5 BY 1);
	 * Setting for legend: no frame around it, centered at top, 2 labels across, 
	   no overall label for legend, black text height 1.5, labels entered in order of appearance;
	 	LEGEND1 NOFRAME POSITION=(CENTER TOP) ACROSS=2 LABEL=NONE 
			VALUE=(COLOR=Black HEIGHT=10pt "Label for Line 1" "Label for Line 2");
	 *  Specification for first line -- line=1 means solid;
	 	SYMBOL1 VALUE=DOT    COLOR=BLACK HEIGHT=1.5 LINE=1 WIDTH=2;
	 * Specification for second line -- line=3 means dashed;
	 	SYMBOL2 VALUE=CIRCLE COLOR=BLUE  HEIGHT=1.5 LINE=3 WIDTH=2;
	 * Plot Y*X=separate line grouping, use axes and legends defined above;
	 	PLOT Y*X=group / VAXIS=AXIS1 HAXIS=AXIS2 LEGEND=LEGEND1;  
RUN; QUIT; TITLE1;	



*******************************************************************************
*******				STRING VARIABLE TRANSFORMATIONS						*******
******************************************************************************;

* Transformations must happen within a DATA step: use DATA + SET;
* If you forget the SET, you will create an empty file!;
DATA &datafile.; SET &datafile.;

* You should define the length of a string variable in a DATA step;
* Otherwise, it uses the length of the first case -- and then truncates!;
	LENGTH newstringvar $10;

* Extracting characters from a string with SUBSTR function;
* First define lengths of two new string variables as 3 and 7 characters;
* "Areacode" is extracted from "fullnum" starting at 1 for 3 characters;
* "Phonenum" is extracted from "fullnum" starting at 4 for 7 characters;
* The character to begin with is required, but the ending character is not;
	LENGTH areacode $3 phonenum $7;
	areacode = SUBSTR(fullnum, 1, 3);
	phonenum = SUBSTR(fullnum, 4, 7);
	LABEL areacode= "areacode: 1st 3 digits from fullnum"
		  phonenum= "phonenum: last 7 digits from fullnum";

* Combining (concatenating) separate string variables with CAT functions;
* Here is how to go backwards from the example above;
* "Fullnum" has the areacode and phonenum added together;
* "Fullnumd" also has a "-" character added between them;
	LENGTH fullnum $10 fullnumd $11;
	fullnum  = CAT(areacode, phonenum);
	fullnumd = CATX("-", areacode, phonenum);
	LABEL fullnum=  "fullnum: Area Code + Phone Number"
		  fullnumd= "fullnumd: Area Code + Phone Number with dash";

* Need to delete leading or trailing blanks? STRIP function;
	varnoblank = STRIP(stringvar);
	LABEL varnoblank= "varnoblank: StringVar with blanks removed";

* Convert FROM numeric TO string;
* First define length of new string variable as 2 characters;
* Then convert numeric (with "F2.0" format) into new string variable;
	LENGTH svar1 $2;
	svar1 = PUT(nvar1, 2.0);
	LABEL svar1= "svar1: String Variable out of Numeric";

* Convert FROM string TO numeric;
	nvar1 = INPUT(svar1, 5.0);
	LABEL nvar1= "nvar1: Numeric Variable out of String";

* The IN function searches a variable for a series of possible values;
	IF svar1 IN("a", "b", "c") THEN flag = 1; ELSE flag = 0;
	LABEL flag= "flag: Svar1=a, b, or c (0=no, 1=yes)";	
* Make a new string variable, same idea;
	LENGTH flag $6;
	IF svar1 IN("a", "b", "c") THEN flag = "ABC"; ELSE flag = "NOTABC";

* The MISSING function returns the number of missing responses;
	smiss10 = MISSING(var1--var10);
	LABEL smiss10= "smiss10: Number Missing of var1-var10";

* The INDEX function can be used to find where a character is;
* The UPCASE function converts the string to uppercase;
* Without UPCASE, ankle ~= ANKLE;
* "ankflag" will take on the position where 'ANKLE' is found in the variable;
	ankflag = INDEX(UPCASE(injury), 'ANKLE');
	LABEL ankflag= "ankflag: Position of 'ankle' in 'injury'";

* Finding character strings identified by delimiters;
* Here, we extract strings on unknown length separated by "-";
* "School" is extracted from id1 starting at position 1, until "-";
* "Pupil" is extracted from id1 starting at "-" until end of variable;
* The -1 and +1 tell it to skip the "-" from the first or last part; 
	LENGTH school pupil $3;
	school = SUBSTR(id1, 1, INDEX(id1, '-') -1);
	pupil  = SUBSTR(id1,    INDEX(id1, '-') +1);
	LABEL school= "school: Part of ID1 before dash"
		  pupil=  "pupil: Part of ID1 after dash";
 
* Don't forget the "run" that closes the data step;
RUN;
 

*******************************************************************************
*******				DATE VARIABLE TRANSFORMATIONS						*******
******************************************************************************;

* To read in 2-digit years correctly, may need to change global OPTIONS;
* Here, a year of "09" will be read as "1909";
OPTIONS YEARCUTOFF=1900;
* Here, a year of "09" will be read as "2009";
OPTIONS YEARCUTOFF=1910;

* Transformations must happen within a DATA step: use DATA + SET;
* If you forget the SET, you will create an empty file!;
DATA &datafile.; SET &datafile.;

* Formatting date variable to display as dates;
	FORMAT datevar MMDDYY10.;
	LABEL datevar "datevar: MM/DD/YYYY";

* Creating a date variable from separate month, day, year variables;
	mydate1 = MDY(01, 01, 98);
	mydate2 = MDY(monthvar, dayvar, yearvar);
	
* Extracting parts out of date variable;
	dayweek  = WEEKDAY(mydate1);	* returns 1-7 for day of week;
	daymonth = DAY(mydate1);		* returns 1-31 for day of month;
	monthvar = MONTH(mydate1);		* returns 1-12 for month of year;
	yearvar  = YEAR(mydate1);		* returns year;

* Finding difference between two dates when given as dates -- here, "age";
* The first argument is unit time: years, months, weeks, hours, seconds;
	age = INTCK('year',birthdate, nowdate);
	LABEL age= "age: Age in Years (nowdate - birthdate)";

* Finding difference between two dates when given in pieces -- here, "age";
* Rounds to nearest interval given (here, years);
	age =  INTCK('year',(MDY(bmonth,bday,byear)),(MDY(nowmonth,nowday,nowyear)));
	LABEL age= "age: Age in Years (birthdate - now)";
	
* Figuring out dates based on reference date and distance past;
* INTNX returns date of day 21 days later than StartDate;
	Day21  = INTNX('day',StartDate,21);
	LABEL Day21= "Day21: Day of 21-day followup from StartDate";

* Don't forget the "run" that closes the data step;
RUN;
